################################################################################
#
#       This is part of CATBox (Combinatorial Algorithm Toolbox) 
#       version 1.2.7 from 1/25/2021. You can find more information at
#       http://schliep.org/CATBox/
#
#	file:   LabelCorrecting.pro
#	author: Torsten Pattberg (torsten.pattberg@klf.de)
#               Alexander Schliep (alexander@schliep.org)
#
#       Copyright (C) 1998-2020, Winfried Hochstaettler and Alexander Schliep.
#	(C) 2010 Springer Verlag.
#       
#	This is part of the Springer textbook "CATBox - an interactive course in 
#       discrete optimization".
#
#       All rights reserved. Do not distribute without written permission. 
#
#    
#
#
################################################################################

#### Options #################################################

breakpoints = [7]
interactive = [1]
graphDisplays = 1
about = """<HTML>
<HEAD>
<TITLE>Shortest Path - Label Correcting</TITLE>
</HEAD>
<BODY>

This algorithm finds a shortest path tree from the source s
to all other vertices in a connected, undirected graph.

</BODY></HTML>
"""

root = None


class MyGraphInformer(WeightedGraphInformer):

    def VertexInfo(self,v):
        if v == root:
            return "Vertex %d is root of tree" % (v)
        elif dist[v] == gInfinity:
            return "Vertex %d is not in tree" % (v)
        elif self.G.edgeWeights[0].QInteger():
            return "Vertex %d has predecessor %d and distance %d" % (v,
                                                     pred[v],dist[v])
        else:
            return "Vertex %d has predecessor %d and distance %.2f" % (v,
                                                     pred[v],dist[v])

def pickCallback(v):
    A.SetVertexAnnotation(v,"s")
    global root
    root = v

PickVertex   = lambda f=pickCallback: self.PickVertex(1,None,f)
Neighborhood = lambda v, a=A, g=G: AnimatedNeighborhood(a,g,v,["red"])

length       = G.edgeWeights[0]
pred         = AnimatedPredecessor(A)
Q            = AnimatedVertexQueue(A,"blue","red")
dist         = {}

A.SetAllVerticesColor("#EEEEEE")

for v in G.vertices:
	dist[v] = gInfinity
	pred[v] = None

A.RegisterGraphInformer(MyGraphInformer(G,"length"))

