################################################################################
#
#       This is part of CATBox (Combinatorial Algorithm Toolbox) 
#       version 1.2.7 from 1/25/2021. You can find more information at 
#       http://schliep.org/CATBox/
#
#	file:   FindPath.pro
#	author: Alexander Schliep (schliep@zpr.uni-koeln.de)
#
#       Copyright (C) 1998-2020, Winfried Hochstaettler and Alexander Schliep.
#	(C) 2010 Springer Verlag.
#       
#	This is part of the Springer textbook "CATBox - an interactive course in 
#       discrete optimization".
#
#       All rights reserved. Do not distribute without written permission. 
#	
#
#
#
################################################################################

#### Options #################################################

breakpoints = [9]
interactive = [1,2]
graphDisplays = 1
self.NeededProperties({'Connected':1,'EdgeWeights':1, 'NonNegativeEdgeWeights':1})
about = """<HTML>
<HEAD>
<TITLE>Shortest Path - Find Path</TITLE>
</HEAD>
<BODY>

This algorithm, a variation of Dijkstra's algorithm, finds a shortest path from 
the source to the sink in a graph.

</BODY></HTML>
"""
root = None

class MyGraphInformer(WeightedGraphInformer):
    def VertexInfo(self,v):
        if v == root:
            return "Vertex %d is root of tree" % v
        elif dist[v] == gInfinity:
            return "Vertex %d is not in tree" % v
        elif self.G.edgeWeights[0].QInteger():
            return "Vertex %d has predecessor %d and distance %d" % (v,pred[v],dist[v])
        else:
            return "Vertex %d has predecessor %d and distance %.2f" % (v,pred[v],dist[v])


def pickCallback(A,v,type):
    if type == "s":
        A.SetVertexAnnotation(v,type,"green")
    else:
        A.SetVertexAnnotation(v,type,"red")
    return

def pickCallbackSource(v):
    pickCallback(A, v, "s")
    global root
    root = v

PickSource         = lambda f=pickCallbackSource: self.PickVertex(1,None,f)
pickCallbackSink   = lambda v, type="t", a=A: pickCallback(a,v,type)
PickSink           = lambda f=pickCallbackSink: self.PickVertex(G.Order(),None,f)
#ShowPath           = lambda u,v: showPathByPredecessorArray(u,v,pred,A,"yellow")
Neighborhood       = lambda v,a=A,g=G: AnimatedNeighborhood(a,g,v,['#EE9900',"red"])

def ShowPath(u,v):
    w = v
    path = []
    while w is not None:
        path.append(w)
        w = pred[w]
    id = A.HighlightPath(path, "yellow")
    #print id

class MyAnimatedVertexPriorityQueue(AnimatedVertexPriorityQueue):
    def DeleteMin(self):
        v = AnimatedVertexPriorityQueue.DeleteMin(self)
        if v and pred[v]:
            A.SetEdgeColor(pred[v], v, 'red')        
        return v

length       = G.edgeWeights[0]
pred         = AnimatedPredecessor(A, predColor='#EE9900')
PQ           = MyAnimatedVertexPriorityQueue(A, colorOff='red')
dist         = AnimatedVertexLabeling(A)

for v in G.vertices:
    pred[v] = None
    dist[v] = gInfinity

A.RegisterGraphInformer(MyGraphInformer(G,"length"))
