################################################################################
#
#       This is part of CATBox (Combinatorial Algorithm Toolbox) 
#       version 1.2.7 from 1/25/2021. You can find more information at 
#       http://schliep.org/CATBox/
#
#	file:   NegativeCircuits.pro
#	author: Alexander Schliep (alexander@schliep.org)
#
#       Copyright (C) 1998-2020, Winfried Hochstaettler and Alexander Schliep.
#	(C) 2010 Springer Verlag.
#       
#	This is part of the Springer textbook "CATBox - an interactive course in 
#       discrete optimization".
#
#       All rights reserved. Do not distribute without written permission. 
#
#       
#
#
################################################################################

#### Options #################################################

breakpoints = [14]
interactive = [1]
graphDisplays = 1
#self.NeededProperties({'EdgeWeights':2, 'Directed':1})
about = """<HTML>
<HEAD>
<TITLE>Shortest Path - Negative Circuits</TITLE>
</HEAD>
<BODY>

This algorithm finds a negative circuit in a graph,
if one exists.

</BODY></HTML>
"""

root = None


class MyGraphInformer(WeightedGraphInformer):

    def VertexInfo(self,v):
        if v == root:
            return "Vertex %d is root of tree." % v
        elif dist[v] == gInfinity:
            return "Vertex %d is not in tree" % (v)
        elif self.G.edgeWeights[0].QInteger():
            if not pred[v]:
                return "Vertex %d has no predecessor and distance %d" % (v,
                                                     dist[v])
            else:
                return "Vertex %d has predecessor %d and distance %d" % (v,
                                                     pred[v],dist[v])
        else:
            if not pred[v]:
                return "Vertex %d has no predecessor and distance %.2f" % (v,
                                                     dist[v])
            else:
                return "Vertex %d has predecessor %d and distance %.2f" % (v,
                                                     pred[v],dist[v])

def pickCallback(v):
    A.SetVertexAnnotation(v,"s")
    global root
    root = v

PickVertex   = lambda f=pickCallback: self.PickVertex(1,None,f)
Neighborhood = lambda v, a=A, g=G: AnimatedNeighborhood(a,g,v,["red"])

length = G.edgeWeights[0]
pred = AnimatedPredecessor(A)
NOW = AnimatedVertexQueue(A,"blue","grey")
NEXT = AnimatedVertexQueue(A,"red","grey")

dist = {}
marked = AnimatedVertexLabeling(A, color="yellow")
circuit = AnimatedSubGraph(G,A,"yellow")

A.SetAllVerticesColor("green")

for v in G.vertices:
	dist[v] = gInfinity
	pred[v] = None

A.RegisterGraphInformer(MyGraphInformer(G,"length"))

