################################################################################
#
#       This is part of CATBox (Combinatorial Algorithm Toolbox) 
#       version 1.2.7 from 1/25/2021. You can find more information at 
#       http://schliep.org/CATBox/
#
#       file:   FindPath.pro
#       author: Alexander Schliep (schliep@zpr.uni-koeln.de)
#               Winfried Hochstaettler (winfried.hochstaettler@fernuni-hagen.de)
#
#       Copyright (C) 1998-2020, Winfried Hochstaettler and Alexander Schliep.
#	(C) 2010 Springer Verlag.
#       
#	This is part of the Springer textbook "CATBox - an interactive course in 
#       discrete optimization".
#
#       All rights reserved. Do not distribute without written permission. 
#
#
#
#
################################################################################


#### Options #################################################

breakpoints = [11,28]
interactive = [1,2]
graphDisplays = 1
self.NeededProperties({'Undirected':1, 'Connected':1,'EdgeWeights':1, 'NonNegativeEdgeWeights':1})
about = """<HTML>
<HEAD>
<TITLE>Shortest Path - Find Path</TITLE>
</HEAD>
<BODY>

This algorithm, a variation of Dijkstra's algorithm, finds a shortest path from 
the source to the sink in a graph.

</BODY></HTML>
"""
s = None
t = None
Bridge       = [None,None]
global PathId
PathId = None

s = t = None

class MyGraphInformer(WeightedGraphInformer):
    def VertexInfo(self,v):
        if v == s:
            return "Vertex %d is source" % v
        elif v == t:
            return "Vertex %d is sink" % v
        elif dist_s[v] == gInfinity:
            if dist_t[v] == gInfinity:
                return "Vertex %d is in neither tree" % v
            else:
                if self.G.edgeWeights[0].QInteger():
                    return "Vertex %d has predecessor %d and distance %d from sink" % (
                        v,pred_t[v],dist_t[v])
                else:
                    return "Vertex %d has predecessor %d and distance %.2f from sink" % (
                        v,pred_t[v],dist_t[v])
        else:
            if dist_t[v] == gInfinity:
                if self.G.edgeWeights[0].QInteger():
                    return "Vertex %d has predecessor %d and distance %d from source" % (
                        v,pred_s[v],dist_s[v])
                else:
                    return "Vertex %d has predecessor %d and distance %.2f from source" % (
                        v,pred_s[v],dist_s[v])
            else:
                if self.G.edgeWeights[0].QInteger():
                    return "Vertex %d has predecessor %d (%d) and distance %d (%d) from source (sink)" % (
                        v,pred_s[v],pred_t[v],dist_s[v],dist_t[v])
                else:
                    return "Vertex %d has predecessor %d (%d) and distance %.2f (%.2f) from source (sink)" % (
                        v,pred_s[v],pred_t[v],dist_s[v],dist_t[v])


def pickCallback(A,v,type):
    if type == "s":
        A.SetVertexAnnotation(v,type,"green")
    else:
        A.SetVertexAnnotation(v,type,"red")
    return

def pickCallbackSource(v):
    pickCallback(A, v, "s")
    global s
    s = v

PickSource = lambda f=pickCallbackSource: self.PickVertex(1,None,f)
pickCallbackSink = lambda v, type="t", a=A: pickCallback(a,v,type)
PickSink = lambda f=pickCallbackSink: self.PickVertex(G.Order(),None,f)
Neighborhood = lambda v,a=A,g=G: AnimatedNeighborhood(a,g,v,['#EE9900','red','#009000'])

def SetBridge(u,v):
    global Bridge
    global PathId
    if PathId != None:
        A.HidePath(PathId)
    Bridge=[u,v]
    w = Bridge[0]
    path_s = []
    while w is not None:
        path_s.append(w)
        w = pred_s[w]
    w = Bridge[1]
    path_t = []
    while w is not None:
        path_t.append(w)
        w = pred_t[w]
    path=[]
    l=len(path_s)
    for i in range(l):
       path.append(path_s[l-1-i])
    path=path+path_t
    PathId = A.HighlightPath(path, "grey")

def ShowPath(Bridge):
    w = Bridge[0]
    path_s = []
    while w is not None:
        path_s.append(w)
        w = pred_s[w]
    w = Bridge[1]
    path_t = []
    while w is not None:
        path_t.append(w)
        w = pred_t[w]
    path=[]
    l=len(path_s)
    for i in range(l):
       path.append(path_s[l-1-i])
    path=path+path_t
    id = A.HighlightPath(path, "yellow")

class MyAnimatedVertexPriorityQueue(AnimatedVertexPriorityQueue):
    def __init__(self, theAnimator, root, colorOn=g.cOnQueue, colorOff=g.cRemovedFromQueue):
        """ theAnimator will usually be the GraphDisplay(Frame/Toplevel) """
        PriorityQueue.__init__(self)
        self.Animator = theAnimator
        self.ColorOn = colorOn
        self.ColorOff = colorOff
        self.lastRemoved = None
        self.root = root

    def DeleteMin(self):
        v = AnimatedVertexPriorityQueue.DeleteMin(self)
        if v and self.root=="s" and pred_s[v] != None:
            A.SetEdgeColor(pred_s[v], v, 'red')
        if v and self.root=="t" and pred_t[v] != None:
            A.SetEdgeColor(pred_t[v], v, '#009000')
        return v

length       = G.edgeWeights[0]
currentlength = gInfinity
pred_s       = AnimatedPredecessor(A, predColor='#EE9900')
PQ_s         = MyAnimatedVertexPriorityQueue(A, "s", colorOn='blue', colorOff='red')
dist_s       = VertexLabeling() #AnimatedVertexLabeling(A)
permanent_s  = VertexLabeling()
pred_t       = AnimatedPredecessor(A, predColor='#99EE00')
PQ_t         = MyAnimatedVertexPriorityQueue(A, "t", colorOn='#87ceff', colorOff='#009000')
dist_t       = VertexLabeling() #AnimatedVertexLabeling(A)
permanent_t  = VertexLabeling()

currentlength = gInfinity

for v in G.vertices:
    pred_s[v] = None
    dist_s[v] = gInfinity
    pred_t[v] = None
    dist_t[v] = gInfinity
    permanent_s[v] = FALSE
    permanent_t[v] = FALSE

A.SetAllVerticesColor("#DDDDDD")

A.RegisterGraphInformer(MyGraphInformer(G,"length"))
